from resources.lib import connect
import sys
import urllib.parse
import xbmc, xbmcaddon, xbmcgui, xbmcplugin


__addon__ = xbmcaddon.Addon()

base_url = sys.argv[0]
__addon_handle__ = int(sys.argv[1])
args = urllib.parse.parse_qs(sys.argv[2][1:])

xbmcplugin.setContent(__addon_handle__, 'videos')


def build_url(query):
    """Get the addon url based on Kodi request"""

    return f"{base_url}?{urllib.parse.urlencode(query)}"

def retrieve_channel_list(session):
    """Show the entire channel list"""

    channel_listing = []
    channel_dict = connect.get_channels(session)
    for i in channel_dict.keys():
        url = build_url({'mode': 'live', 'id': i})
        li = xbmcgui.ListItem(channel_dict[i]["name"])
        li.setArt({"thumb": channel_dict[i]['img']})
        channel_listing.append((url, li, False))

    xbmcplugin.addDirectoryItems(__addon_handle__, channel_listing, len(channel_listing))
    xbmcplugin.endOfDirectory(__addon_handle__)

def play_channel(session, channel_id):
    """Pass the playback+license URLs to the Kodi Player"""

    stream_url = connect.get_channel_playlist(session, channel_id)
    li = xbmcgui.ListItem(path=stream_url)
    license_url = "https://vmxdrmfklb1.sfm.t-online.de:8063/"

    li.setProperty('inputstream.adaptive.license_key', license_url + "|deviceId=" + session["deviceId"] + "|R{SSM}|")
    li.setProperty('inputstream.adaptive.license_type', "com.widevine.alpha")

    li.setProperty('inputstream', 'inputstream.adaptive')
    li.setProperty('inputstream.adaptive.manifest_type', 'mpd')
    li.setProperty("IsPlayable", "true")

    li.setInfo("video", {"title": "CHANNEL"})

    xbmcplugin.setResolvedUrl(__addon_handle__, True, li)

    xbmc.Player().play(item=stream_url, listitem=li)

def router(item):
    """Router function calling other functions of this script"""

    # LOGIN
    __login = __addon__.getSetting("username")
    __password = __addon__.getSetting("password")
    session = connect.login_process(__login, __password)

    params = dict(urllib.parse.parse_qsl(item[1:]))

    if params:

        # LIVE TV CHANNEL
        if params.get("mode") == "live" and params.get("id"):
            play_channel(session, params["id"])

        # LIVE TV CHANNEL LIST
        if params.get("mode") == "live":
            retrieve_channel_list(session)

    else:
        # MAIN MENU
        main_listing = []
        for mode in [("live", "Live TV")]:
            url = build_url({'mode': mode[0]})
            li = xbmcgui.ListItem(mode[1])
            main_listing.append((url, li, True))

        xbmcplugin.addDirectoryItems(__addon_handle__, main_listing, len(main_listing))
        xbmcplugin.endOfDirectory(__addon_handle__)


if __name__ == "__main__":
    router(sys.argv[2])